import logging
import threading
import time
from datetime import datetime

from eventmanager import ClusterEventManager
from jobfiller import JobFiller
from productionjob import ProductionJob

logger = logging.getLogger(__name__)


class MachineDoctor(object):
    """
    Try and SSH into the machine and see whats up.  If we can't
    get to it and reboot a sitter than decomission it.

    NOTE: We should do some SERIOUS rate limiting here.
    If we just have a 10 minute network hiccup we *should*
    try and replace those machines, but we should continue
    to check for the old ones for *A LONG TIME*
    to see if they come back.  After that formally decomission
    them.  If they do come back after we've moved their jobs around
    then simply remove the jobs from the machine and add them
    to the idle resources pool.
    """
    def __init__(self, sitter):
        self.sitter = sitter
        self.state = self.sitter.state
        self.thread = None

    def start(self):
        self.thread = threading.Thread(target=self._run,
                                           name="MachineDoctor")
        self.thread.start()

    def _run(self):
        while True:
            start_time = datetime.now()
            logger.debug("Begin machine doctor run.  Unreachables: %s"
                         % (self.state.unreachable_machines))
            try:
                self._check_for_unreachable_machines()
                self._turn_off_overflowed_jobs()
                self._enforce_idle_machine_limits()
            except:
                # Der?  Not sure what this could be...
                import traceback
                traceback.print_exc()
                logger.error(traceback.format_exc())

            time_spent = datetime.now() - start_time
            sleep_time = self.sitter.stats_poll_interval - \
                time_spent.seconds
            logger.debug(
                "Finished Machine Doctor run. " +
                "Time_spent: %s, sleep_time: %s" % (
                    time_spent,
                    sleep_time))

            if sleep_time > 0:
                time.sleep(sleep_time)

    def _check_for_unreachable_machines(self):
                """
                Try and fix unreachable machines
                """
                for machine, monitor in self.state.unreachable_machines:
                    # Check if we've aready launched a redeploy job to
                    # this machine, if so skip it
                    found = False
                    for job in self.state.repair_jobs:
                        for fillers in job.fillers.values():
                            for filler in fillers:
                                for jobmachine in filler.machines:
                                    if machine == jobmachine:
                                        found = True
                                        break
                    if found:
                        continue

                    ClusterEventManager.handle("Attempting to redeploy to %s" %
                                               machine)

                    # Build a 'fake' job for the doctor to run
                    job = ProductionJob(
                        "",
                        {'name': 'Machine Doctor Redeployer'},
                        {
                            machine.config.shared_fate_zone: {
                                'mem': machine.config.mem,
                                'cpu': machine.config.cpus
                                }
                            }, None)

                    job.sitter = self.sitter
                    self.state.repair_jobs.append(job)

                    def post_filler(success):
                        if success:
                            logger.info("Successful redeploy of %s!" % machine)
                        else:
                            logger.info(
                                "Redeploy failed! Decomissioning %s" % machine)

                            # Decomission time!
                            ClusterEventManager.handle(
                                "Decomissioning %s" % machine)

                            self.sitter.decomission_machine(machine)

                        # Can happen if machine is decomissioned by
                        # somebody else while we were redeploying.
                        if (machine, monitor) in \
                                self.state.unreachable_machines:
                            self.state.unreachable_machines.remove(
                                (machine, monitor))

                        self.state.repair_jobs.remove(job)

                    filler = JobFiller(1, job, machine.config.shared_fate_zone,
                                       raw_machines=[machine],
                                       post_callback=post_filler,
                                       fail_on_error=True)

                    job.fillers[machine.config.shared_fate_zone] = [filler]

                    filler.start()

    def _turn_off_overflowed_jobs(self):
        """
        Turn off any overflowed jobs
        """
        did_overflow_reduction = False
        for jobname, zone_overflow in self.state.job_overflow.items():
            for zone, count in zone_overflow.items():
                if count <= 0:
                    continue

                did_overflow_reduction = True

                ClusterEventManager.handle(
                    "Detected job overflow:" +
                    "Job: %s, Zone: %s, Count: %s" % (jobname,
                                                      zone,
                                                      count))

                decomissioned = 0
                for machine in self.state.machines_by_zone[zone]:
                    if decomissioned == count:
                        break

                    for task in machine.get_running_tasks():
                        if task['name'] == jobname:
                            while jobname in [
                                t['name'] for t in \
                                    machine.get_running_tasks()]:

                                ClusterEventManager.handle(
                                    "Stopping %s on %s" % (
                                        jobname,
                                        str(machine)))

                                machine.datamanager.stop_task(jobname)
                                machine.datamanager.reload()

                                decomissioned += 1
                                break

                # If we turned off any processess for overflow then we
                # should force an overflow recalculation.  This is to
                # avoid the case of the machinedoctor doing another
                # full loop before the calculator thread has run.  If
                # that happens we might do another round of
                # decomissioning when we haven't yet taken into
                # account the decomissioning we already did
                if did_overflow_reduction:
                    self.state._calculate()

    def _enforce_idle_machine_limits(self):
        """
        Enforce idle machine limits
        """
        if self.state.max_idle_per_zone != -1:
            logger.info("Enforcing an idle limit")
            idle_limit = self.state.max_idle_per_zone
            self.state.max_idle_per_zone = -1
            for zone, machines in self.state.idle_machines.items():
                provider = self.state.provider_by_zone[zone]

                if len(machines) > idle_limit:
                    decomission_targets = [
                        m for m in machines[idle_limit:]]
                    for machine in decomission_targets:
                        self.sitter.decomission_machine(machine)



from datetime import datetime
import logging

logger = logging.getLogger(__name__)


class ClusterEventManager(object):
    inst = None

    def __init__(self):
        self.messages = []

    @classmethod
    def get_events(cls):
        return cls.get_manager().messages

    @classmethod
    def get_manager(cls):
        if not cls.inst:
            cls.inst = cls()

        return cls.inst

    @classmethod
    def handle(cls, msg):
        logger.info(msg)
        now = datetime.now()
        cls.get_manager().messages.insert(0,
                                          "%s: %s" % (str(now),
                                                      msg))



import os
import re
import threading

from tenjin.helpers import *

from eventmanager import ClusterEventManager
from tasksitter.stats_collector import StatsCollector


def strip_html(text):
    return re.sub('<[^<]+?>', '', text)


class ClusterStats(StatsCollector):

    def overview(self, args):
        engine = args['engine']
        data = self.get_metadata()
        data.update(self.get_live_data())
        if "nohtml" not in args:
            return engine.render('cluster_overview.html', {'data': data,
                                                           'pagewidth': 1300})
        else:
            return data

    def get_live_data(self):
        data = {}

        data['events'] = ClusterEventManager.get_events()

        state = self.harness.state

        data['providers'] = state.providers.keys()
        data['machines_by_zone'] = str(state.machines_by_zone)
        data['job_fill'] = str(state.job_fill)
        data['idle_machines'] = str(state.idle_machines)
        data['unreachable_machines'] = [
            str(m) for m in state.unreachable_machines]

        monitors = []
        machines = []

        def serialize_machine(machine):
            machine_data = {}
            machine_data['repr'] = repr(machine)
            machine_data['zone'] = machine.config.shared_fate_zone
            machine_data['bits'] = machine.config.bits
            machine_data['cpus'] = machine.config.cpus
            machine_data['mem'] = machine.config.mem
            machine_data['url'] = machine.datamanager.url
            machine_data['disk'] = machine.config.disk
            machine_data['dns_name'] = machine.config.dns_name
            machine_data['hostname'] = machine.hostname
            machine_data['tasks'] = machine.get_tasks()
            for task in machine_data['tasks'].values():
                # task is running
                if 'monitoring' in task:
                    filenum = int(task.get('num_task_starts', 1))

                    # Counting starts at 0
                    filenum -= 1

                    files = ["stdout", "stderr"]
                    for filename in files:
                        link = strip_html(
                            "%s/logfile?logname=%s.%s&tail=100" % (
                                task['monitoring'],
                                filename,
                                filenum))

                        task[filename] = "<a href='%s'>%s</a>" % (
                            link, filename)

            machine_data['running_tasks'] = machine.get_running_tasks()
            machine_data['is_in_deployment'] = machine.is_in_deployment()
            machine_data['number'] = machine.machine_number
            machine_data['initialized'] = machine.is_initialized()
            machine_data['has_loaded_data'] = machine.has_loaded_data()
            machine_data['pull_failures'] = pull_failures.get(machine, 0)
            machine_data['idle'] = machine in state.idle_machines.get(
                machine_data['zone'], [])

            return machine_data

        for monitor, thread in state.monitors:
            monitor_data = {}
            monitor_data['monitored_machines'] = [
                repr(m) for m in monitor.monitored_machines]
            monitor_data['add_queue'] = [repr(m) for m in monitor.add_queue]
            monitor_data['pull_failures'] = dict([
                (str(k), v) for k, v in monitor.pull_failures.iteritems()])
            monitor_data['failure_threshold'] = monitor.failure_threshold
            monitor_data['number'] = monitor.number
            monitors.append(monitor_data)

            pull_failures = dict(monitor.pull_failures)

            for machine in monitor.monitored_machines:
                machine_data = serialize_machine(machine)
                machines.append(machine_data)

        data['machines'] = machines
        data['monitors'] = monitors

        jobs = []
        check_jobs = state.jobs + state.repair_jobs
        for job in check_jobs:
            job_data = {}
            job_data['name'] = job.name
            job_data['task_configuration'] = job.task_configuration
            job_data['deployment_layout'] = job.deployment_layout
            job_data['deployment_recipe'] = job.deployment_recipe
            job_data['recipe_options'] = job.recipe_options
            job_data['linked_job'] = job.linked_job
            fillers = []
            for filler_list in job.fillers.values():
                for filler in filler_list:
                    filler_data = {}
                    filler_data['zone'] = filler.zone
                    filler_data['num_cores'] = filler.num_cores
                    filler_data['machine_states'] = [
                        (m.hostname, str(m.state)) for m in filler.machines]
                    filler_data['state'] = str(filler.state)
                    fillers.append(filler_data)

            job_data['fillers'] = fillers
            job_data['fill'] = state.job_fill.get(job.name, {})

            fill_machines = state.job_fill_machines.get(job.name, {})
            for zone in fill_machines.keys():
                fill_machines[zone] = [str(m) for m in fill_machines[zone]]

            job_data['fill_machines'] = fill_machines

            job_data['spawning'] = job.currently_spawning
            jobs.append(job_data)

        data['jobs'] = jobs

        load = os.getloadavg()
        data['load_one_min'] = load[0]
        data['load_five_min'] = load[1]
        data['load_fifteen_min'] = load[2]

        py_threads = threading.enumerate()
        alive_thread_names = [t.getName() for t in py_threads]
        threads = {}

        std_threads = ['MachineDoctor', 'MainThread',
                       'Calculator', "HTTPServer"]
        for i in range(self.harness.worker_thread_count):
            std_threads.append("Monitoring-%s" % i)

        for name in std_threads:
            if name in alive_thread_names:
                threads[name] = True
                alive_thread_names.remove(name)
            else:
                threads[name] = False

        other_threads = {}
        for name in alive_thread_names:
            other_threads[name] = True

        data['std_threads'] = std_threads
        data['threads'] = threads
        data['other_threads'] = other_threads
        return data

    def get_metadata(self):
        data = {}
        data['clustersitter_pid'] = os.getpid()
        data['log_location'] = self.harness.log_location
        data['provider_config'] = self.harness.provider_config
        data['dns_provider_config'] = self.harness.dns_provider_config
        data['username'] = self.harness.user
        data['keys'] = self.harness.keys
        data['launch_time'] = str(self.harness.launch_time)
        data['launch_location'] = self.harness.launch_location
        data['start_state'] = self.harness.start_state
        data['logfiles'] = self.harness.logmanager.get_logfile_names()

        return data



import logging
import os
import paramiko
import subprocess

logger = logging.getLogger(__name__)


class DeploymentRecipe(object):
    def __init__(self, hostname, username, keys,
                 post_callback=None, options=None,
                 given_logger=None, dns_hostname=None,
                 launch_location=None):
        self.hostname = hostname
        self.username = username
        self.dns_hostname = dns_hostname
        self.launch_location = launch_location
        self.keys = keys
        if not isinstance(self.keys, list):
            self.keys = [self.keys]

        self.post_callback = post_callback
        self.options = options
        self.logger = logger

        if given_logger:
            self.logger = given_logger

        self.connected = False
        self.connect()

    def connect(self):
        if self.connected:
            return

        self.client = paramiko.SSHClient()
        self.client.set_missing_host_key_policy(
            paramiko.AutoAddPolicy())

        for key in self.keys:
            try:
                self.client.connect(hostname=self.hostname,
                                    username=self.username,
                                    key_filename=key,
                                    timeout=10)
                self.sftp = self.client.open_sftp()
                self.connected = True
                self.logger.info("Connected to %s@%s with %s" % (
                        self.username, self.hostname, key
                        ))
                break
            except:
                self.logger.warn("Couldnt ssh into %s@%s with %s" % (
                        self.username, self.hostname, key
                        ))
                continue

    def run(self, cmd):
        self.logger.info("Running %s on %s" % (cmd, self.hostname))
        output = self.client.exec_command(cmd)
        stdout = output[1]
        stderr = output[2]
        stdout_log = []
        while True:
            line = stdout.readline()
            if not line:
                break

            self.logger.info("Output from (%s): %s" % (cmd, line.strip()))
            stdout_log.append(line)
        self.logger.info("Stderr from (%s): %s" % (cmd, stderr.readlines()))
        return stdout_log

    def put(self, local, remote):
        if os.path.basename(local) != os.path.basename(remote):
            remote += os.path.basename(local)

        try:
            local_hash = subprocess.check_output(
                ['md5sum', local]).split(' ')[0]
        except subprocess.CalledProcessError:
            import traceback
            self.logger.error(traceback.format_exc())
            return

        self.logger.info("Calculated local hash: %s", local_hash)
        remote_hash = self.run("md5sum %s" % remote)
        if remote_hash:
            remote_hash = remote_hash[0].split(' ')[0]
            self.logger.info("Calculated remote hash: %s", remote_hash)

        val = True
        if local_hash != remote_hash:
            self.logger.info(
                "Hashes differ.  Uploading %s to %s" % (local, remote))
            val = self.sftp.put(local, remote)
        else:
            self.logger.info("Hashes equal, not reuploading")

        return val

    def sudo(self, cmd):
        return self.run("sudo bash -c '%s'" % cmd)

    def deploy(self):
        if not self.connected:
            self.logger.warn(
                "Couldn't do deployment: SSH Not connected." +
                "For AWS machines this may be normal, as it can be up to " +
                "2 minutes after they spawn before we can login")

            return False

        self.logger.info("Begin deployment on %s" % self.hostname)
        # 2 tries
        try:
            retval = self.run_deploy()
        except:
            try:
                retval = self.run_deploy()
            except:
                import traceback
                self.logger.error(traceback.format_exc())
                return False

        self.logger.info("Calling post-deply callback for %s" % self.hostname)
        if self.post_callback:
            self.post_callback()

        return retval


class MachineSitterRecipe(DeploymentRecipe):
    def run_deploy(self):
        try:
            self.sudo("apt-get update")

            # Now create the remote directory
            self.run("mkdir -p %s" % remote_dir)

            if self.dns_hostname:
                self.sudo("hostname %s" % self.dns_hostname)
            else:
                self.sudo("hostname %s" % self.hostname)

            self.sudo("apt-get install python-setuptools")
            self.sudo("easy_install cerebrod")

            # Clear any old ones
            self.sudo("pkill -9 -f sitter")

            # ensure we have the log location created
            log_option = ""
            if self.options.get('log_location'):
                self.sudo("mkdir -p %s" % self.options['log_location'])
                log_option = "--log-location=%s" % self.options['log_location']

            # Launch a machine sitter as root
            self.sudo("machinesitter --daemon %s" % (
                    log_option
                    ))
        except:
            import traceback
            traceback.print_exc()
            return False

        return True



import boto
import logging
import os
import socket
import time
from boto import ec2

from clustersitter.machineconfig import MachineConfig
from clustersitter.eventmanager import ClusterEventManager
from machineprovider import MachineProvider

logger = logging.getLogger(__name__)


class AmazonEC2(MachineProvider):
    instance_types = {
        'm1.small': (1, 1700, 160, 32),
        'm1.large': (2, 7500, 850, 64),
        'm1.xlarge': (4, 15000, 1690, 64),
        't1.micro': (1, 613, 0, 64),
        'm2.xlarge': (2, 17100, 420, 64),
        'm2.2xlarge': (4, 34200, 850, 64),
        'm2.4xlarge': (8, 68400, 1690, 64),
        'c1.medium': (2, 1700, 350, 32),
        'c1.xlarge': (8, 7000, 1690, 64)
        }

    def __init__(self, config):
        self.config = config
        self.zones = []
        self.connection_by_zone = {}
        self.regions = []
        self.connections = {}
        self.machines = {}

        # TODO Ensure security group has port 22 permissions
        try:
            logger.info("Download EC2 Region List")
            self.regions = ec2.regions()
        except boto.exception.NoAuthHandlerFound:
            logger.error("Couldn't connect to EC2.  Auth Error.  " +
                         "Ensure AWS crednetials are set in the env " +
                         "(AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY")

            logger.error("Got: %s, %s" % (os.getenv('AWS_ACCESS_KEY_ID'),
                                          os.getenv('AWS_SECRET_ACCESS_KEY')))

        except:
            import traceback
            logger.error("Couldn't connect to EC2.")
            logger.error(traceback.format_exc())

    def usable(self):
        return bool(self.regions)

    def decomission(self, machine):
        """
        machine -- A Monitored Machine
        """
        aws_placement = machine.config.shared_fate_zone.replace(
            'aws-', '')
        conn = self.connection_by_zone[aws_placement]
        instance = machine.config.data
        try:
            conn.terminate_instances([instance.id])
        except:
            import traceback
            logger.warn(traceback.format_exc())
            return False

        return True

    @classmethod
    def config_from_instance(cls, instance):
        perf = cls.instance_types[instance.instance_type]
        ip = socket.gethostbyname(instance.public_dns_name)
        return MachineConfig(instance.public_dns_name,
                             "aws-%s" % instance.placement,
                             cpus=perf[0],
                             mem=perf[1],
                             disk=perf[2],
                             bits=perf[3],
                             data=instance,
                             ip=ip
                             )

    def _get_image_by_type(self, zone, instance_type):
        if self.instance_types[instance_type][3] == 32:
            return self.config[zone]['32b_image_id']
        else:
            return self.config[zone]['64b_image_id']

    def fill_request(self, zone, cpus, mem_per_job=None):
        """
        Ideally what we do now is figure out
        what instance_type combination best fills
        this request.  For simplicity for now
        we'll just spin up one m1.large per cpu.
        """
        # TODO if this fails decomission the machines and return false
        aws_placement = zone.replace('aws-', '')
        conn = self.connection_by_zone[aws_placement]

        # TODO detect this better
        instance_type = 'm1.large'
        ClusterEventManager.handle(
            "Spinning up %s amazon instances..." % cpus)
        reservation = None
        try:
            reservation = conn.run_instances(
                image_id=self._get_image_by_type(aws_placement, instance_type),
                key_name=self.config[aws_placement]['key_name'],
                security_groups=self.config[aws_placement]['security_groups'],
                min_count=cpus,
                max_count=cpus,
                instance_type=instance_type,
                placement=aws_placement,
                monitoring_enabled=False
                )
        except:
            import traceback
            logger.error(traceback.format_exc())
            return False

        instance_ids = [i.id for i in reservation.instances]
        logger.info("Reservation made for %s instances of type %s" % (
                cpus, instance_type))
        logger.info("Ids: %s" % instance_ids)
        done = False
        instances = []
        failure_count = 0
        while not done:
            # Add logging here
            # Don't wait for all of them, incase some don't come up
            available = 0
            reservation = None
            try:
                reservation = conn.get_all_instances(instance_ids)[0]
            except:
                failure_count += 1
                if failure_count > 5:
                    logger.error(traceback.format_exc())
                    return False
                continue
            instances = reservation.instances
            all_found = True
            for instance in instances:
                if not (instance.public_dns_name and
                        instance.state == 'running'):
                    all_found = False
                else:
                    available += 1

            logger.info("%s of %s instances are ready" % (available,
                                                           len(instances)))
            done = all_found
            if not done:
                time.sleep(1)

        logger.info("All instances up, returning from AWS deploy routine")

        return [AmazonEC2.config_from_instance(i) for i in instances]

    def get_machine_list(self):
        self._initialize_connections()
        machine_list = []
        for conn in self.connections.values():
            reservations = conn.get_all_instances()
            if reservations:
                for reservation in reservations:
                    for instance in reservation.instances:
                        sf_zone = "aws-%s" % instance.placement
                        if sf_zone not in self.machines:
                            self.machines[sf_zone] = []

                        if (instance.public_dns_name and
                            instance.state == 'running'):
                            self.machines[sf_zone].append(instance)
                            machine_list.append(
                                AmazonEC2.config_from_instance(instance))

        return machine_list

    def _initialize_connections(self):
        for region in self.regions:
            if not region.name in self.connections:
                logger.info("Connecting to %s" % region.name)
                self.connections[region.name] = ec2.connect_to_region(
                    region.name)

    def get_all_shared_fate_zones(self):
        if self.zones:
            return self.zones

        #self.zones = [u'eu-west-1a', u'eu-west-1b', u'eu-west-1c', u'sa-east-1a', u'sa-east-1b', u'us-east-1a', u'us-east-1b', u'us-east-1c', u'us-east-1d', u'ap-northeast-1a', u'ap-northeast-1b', u'us-west-2a', u'us-west-2b', u'us-west-1b', u'us-west-1c', u'ap-southeast-1a', u'ap-southeast-1b']

        self._initialize_connections()
        for conn in self.connections.values():
            zones = conn.get_all_zones()
            for zone in zones:
                self.connection_by_zone[zone.name] = conn
                self.zones.append(zone.name)

        return ["aws-%s" % s for s in self.zones]



"""
A class which knows how to pull information out of a process harness
and process.  Exposes a simple interface to get all this information.
"""
import os
import socket
import threading


class StatsCollector(object):
    """
    Collect various kinds of statistics about running the child process.
    """
    def __init__(self, harness):
        self.hostname = socket.gethostname()
        self.harness = harness
        self.thread = None

    def start(self):
        """
        Public interface to start the collection thread
        """
        self.thread = threading.Thread(target=self._start_collecting)
        self.thread.start()

    def _start_collecting(self):
        """
        Collect stats as the process is running.
        """
        pass

    def get_live_data(self):
        """
        Return live statistics about the harness and child process
        """
        data = {
            'process_start_time': str(self.harness.child_proc.start_time),
            'num_task_starts': self.harness.start_count,
            }

        for constraint, count in self.harness.violations.items():
            data['violated_%s' % constraint] = count

        data['cpu_usage'] = self.harness.child_proc.cpu_usage
        data['mem_usage_vmem'] = self.harness.child_proc.mem_usage[0]
        data['mem_usage_res'] = self.harness.child_proc.mem_usage[1]
        data['system_usage'] = self.harness.child_proc.system_usage

        return data

    def get_metadata(self):
        """
        Return fixed metadata about the child process
        """
        # Collect version data
        version_file = os.path.join(self.harness.launch_location, 'VERSION')
        file_version = None
        if os.path.exists(version_file):
            filedata = open(version_file)
            file_version = filedata.read()
            filedata.close()

        dir_version = os.path.basename(self.harness.launch_location)

        data = {'child_pid': self.harness.child_proc.pid,
                'task_start_time': str(self.harness.task_start),
                'max_restarts': self.harness.max_restarts,
                'command': self.harness.command,
                'restart': self.harness.restart,
                'file_version': file_version,
                'dir_version': dir_version,
                'launch_location': self.harness.launch_location,
                'constraints': ','.join(
                [str(c) for c in self.harness.constraints])
                }

        return data



from setuptools import setup

setup(name='cerebrod',
      version='1.0.1',
      description='A Cluster Management System',
      author='Zach Goldberg',
      author_email='zach@zachgoldberg.com',
      url='http://zachgoldberg.com/',
      download_url='https://github.com/ZachGoldberg/Cerebro',
      zip_safe=True,
      package_data={
        '': ['templates/*.html', 'sittercommon/templates/*.html'],
         },
      packages=[
        'tasksitter',
        'machinesitter',
        'sittercommon',
        'machineconsole',
        'clustersitter',
        'clustersitter.providers',
        ],
      package_dir={
        'tasksitter': 'src/tasksitter',
        'machinesitter': 'src/machinesitter',
        'sittercommon': 'src/sittercommon',
        'clustersitter': 'src/clustersitter',
        'machineconsole': 'src/machineconsole'
        },
      entry_points={
        'console_scripts': [
            'cerebrod = clustersitter.main:main',
            'clustersitter = clustersitter.main:main',
            'tasksitter = tasksitter.main:main',
            'machinesitter = machinesitter.main:main',
            'machineconsole = machineconsole.main:main',
            ]
        },
      install_requires=[
        'boto',
        'dynect_client',
        'paramiko',
        'requests',
        'tenjin',
        ],
     )



import os
import socket
from tasksitter.stats_collector import StatsCollector


class MachineStats(StatsCollector):

    def get_live_data(self):
        data = {}
        hostname = socket.gethostname()
        for task_name, task in self.harness.tasks.items():
            running = bool(task.is_running())
            data["%s-running" % task.name] = running
            if not running:
                data["%s-start" % task.name] = \
                    "<a href='http://%s:%s/start_task?task_name=%s'>start</a>" % (
                    hostname,
                    self.harness.http_monitor.port,
                    task.name)
            else:
                data["%s-stop" % task.name] = \
                    "<a href='http://%s:%s/stop_task?task_name=%s'>stop</a>" % (
                    hostname,
                    self.harness.http_monitor.port,
                    task.name)
                location = "http://%s:%s" % (
                    hostname,
                    task.http_monitoring_port)
                data["%s-monitoring" % task.name] = "<a href='%s'>%s</a>" % (location,
                                                                           location)

        load = os.getloadavg()
        data['load_one_min'] = load[0]
        data['load_five_min'] = load[1]
        data['load_fifteen_min'] = load[2]

        return data

    def get_metadata(self):
        data = {}
        data['hostname'] = socket.gethostname()
        data['machinesitter_pid'] = os.getpid()
        data['launch_location'] = self.harness.launch_location
        data['log_location'] = self.harness.log_location
        data['task_sitter_starting_port'] = self.harness.task_sitter_starting_port
        data['machine_sitter_starting_port'] = self.harness.machine_sitter_starting_port
        data['task_definition_file'] = self.harness.task_definition_file
        for task_name, task in self.harness.tasks.items():
            data["%s-name" % task.name] = task.name
            data["%s-command" % task.name] = task.command

        return data
